@@ -1,25 +1,28 @@
 # -*- coding: utf-8 -*-
 """
 -------------------------------------------------
-   File Name：     mysql
-   Description :
-   Author :        Asdil
-   date：          2021/4/1
+   File Name：     db_mysql
+   Description :  mysql 连接池
+   Author :       jpl
+   date：          2021/8/16
 -------------------------------------------------
    Change Activity:
-                   2021/4/1:
+                   2021/8/16:
 -------------------------------------------------
 """
 __author__ = 'Asdil'
-from dbutils.persistent_db import PersistentDB
-from dbutils.pooled_db import PooledDB
+
 import pymysql
+from dbutils.pooled_db import PooledDB
+from dbutils.persistent_db import PersistentDB
 
 
 class Mysql:
     """
     Mysql类用于mysql连接池操作
+    PersistentDB主要为单线程应用提供一个持久的连接，而PooledDB通常可以为多线程应用提供线程池服务
     """
+
     def __init__(self, host, port, user, password, database, pool=1):
         """__init__(self):方法用于
 
@@ -44,7 +47,7 @@ class Mysql:
                 creator=pymysql,  # 使用连接数据库模块
                 maxusage=None,  # 连接超时时间
                 setsession=[],  # 开始会话前执行的操作
-                ping=0,  # ping服务器端查看是否可用
+                ping=1,  # ping服务器端查看是否可用
                 closeable=False,  # 实际上被忽略，供下次使用，再线程关闭时，才会自动关闭链接。如果为True时， conn.close()则关闭链接，
                 # 那么再次调用pool.connection时就会报错，因为已经真的关闭了连接（pool.steady_connection()可以获取一个新的链接
                 threadlocal=None,  # 本线程独享值得对象，用于保存链接对象，如果链接对象被重置
@@ -74,11 +77,13 @@ class Mysql:
         ----------
         sql : str
             sql 语句
-        args : list or tuple
+        args : list or tuple or None
             参数
         Returns
         ----------
         """
+        if args:
+            args = tuple(args)
         conn = self.pool.connection(shareable=False)
         cursor = conn.cursor()
         cursor.execute(sql, args)
@@ -113,7 +118,10 @@ class Mysql:
 
         Parameters
         ----------
-        param : str
+        sql : str
+            sql语句
+        args: list or tuple or None
+            参数
 
         Returns
         ----------
@@ -126,13 +134,30 @@ class Mysql:
         conn.commit()
         conn.close()
 
+    def get_cursor(self):
+        """get_cursor方法用于获取一个游标
+        """
+        conn = self.pool.connection(shareable=False)
+        cursor = conn.cursor()
+        return conn, cursor
+
 
 def optimize_expression(sql, params):
     """optimize_expression方法用于优化sql表达式
     使用方法:
-        sql = "select * from table1 where age=%s and name in (%s)"
-        params = [10, ['Tom', 'Jim']]
-        sql, params = optimize_expression(sql, params)
+    type1:
+        sql = 'update test set age=%s where name in (%s)'
+        sql, parm = optimize_expression(sql, [2, ['Alice', 'Bob']])
+        mysql.excute(sql, parm)
+    type2:
+        sql = 'insert into test value(%s)'
+        sql, parm = optimize_expression(sql, [['Tom', 2]])
+        mysql.excute(sql, parm)
+    type3:
+        sql = 'insert into test values %s'
+        sql, parm = optimize_expression(sql, [['Jack', 2], ['San', 3]])
+        mysql.excute(sql, parm)
+
     Parameters
     ----------
     sql : str
@@ -144,18 +169,21 @@ def optimize_expression(sql, params):
     """
     new_params = []
     codes = []
-    for param in params:
-        if isinstance(param, list):
+    if 'values' in sql.lower():
+        for param in params:
             new_params.extend(list(map(str, param)))
-            codes.append(','.join(['%s']*len(param)))
-        else:
-            new_params.append(param)
-            codes.append('%s')
+            tmp = ','.join(['%s'] * len(param))
+            tmp = f'({tmp})'
+            codes.append(tmp)
+        codes = [','.join(codes)]
+
+    else:
+        for param in params:
+            if isinstance(param, list):
+                new_params.extend(list(map(str, param)))
+                codes.append(','.join(['%s'] * len(param)))
+            else:
+                new_params.append(param)
+                codes.append('%s')
     sql = sql % tuple(codes)
-    return sql, new_params
-
-
-if __name__ == '__main__':
-    db = Mysql(host='127.0.0.1', port=3306, user='root', password='', database='mysql')
-    data = db.select_all("show tables;")
-    print(data)
+    return sql, tuple(new_params)
\ No newline at end of file
